#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Feb 17 15:43:02 2021

@author: anton
"""

from SLiCAP import *
fileName = 'noiseAnalysis'
prj = initProject(fileName)
#makeNetlist(fileName + '.asc', FineName)
i1 = instruction()
i1.setCircuit(fileName + '.cir')

htmlPage('Circuit data')
head2html('Circuit diagram')
img2html(fileName + '.svg', 700, label='schematics', caption = 'Circuit diagram')
netlist2html(fileName + '.cir', label = 'netlist')
elementData2html(i1.circuit)
params2html(i1.circuit)

#Define the instruction parameters
i1.setSource('V1')
i1.setDetector('V_out')
i1.setSimType('symbolic')
i1.setGainType('vi')
i1.setDataType('noise')
# Execute the instruction
result = i1.execute()

# Study the results ###########################################################
# The detector-referred noise spectral density
print('Onoise\t:', result.onoise, '\n')
# The dictionary with the spectral contribution of each noise source to the
# detector-referred noise spectral density
for key in list(result.onoiseTerms.keys()):
    print(key, '\t:', result.onoiseTerms[key], '\n')
    
# If a source has been defined, SLiCAP also calculates the source-referred 
# noise spectral density and the contributions of each noise source to it
print('Inoise\t:', result.inoise, '\n')
# The dictionary with the spectral contribution of each noise source to the
# source-referred noise spectral density
for key in list(result.inoiseTerms.keys()):
    print(key, '\t:', result.inoiseTerms[key], '\n')
    
# Put all of the above on an HTML page ########################################
htmlPage('Numeric noise analysis results')
noise2html(result)

# Simplify the noise expressions ##############################################
result.onoise = sp.simplify(result.onoise)
for key in list(result.onoiseTerms.keys()):
    result.onoiseTerms[key] = sp.simplify(result.onoiseTerms[key])

result.inoise = sp.simplify(result.inoise)
for key in list(result.inoiseTerms.keys()):
    result.inoiseTerms[key] = sp.simplify(result.inoiseTerms[key])

# Put these expressions on the HTML page ######################################
htmlPage('Simplified symbolic noise analysis results')
noise2html(result)

# Find a symbolic expression for the detector-referred RMS noise over a #######
# frequency range from f_min to f_max                                   #######
f_min, f_max = sp.symbols('f_min, f_max')

# Symbolic integration limits are not always supported.########################
try:
    rmsDetNoise_fMin_fmax = rmsNoise(result, 'onoise', f_min, f_max)
except:
    print('Trying the work around:\n')
    # Here is a work around (for this situation) ##############################
    rmsDetNoise_f = rmsNoise(result, 'onoise', 0, ini.frequency)
    rmsDetNoise_fMin_fmax = sp.simplify(sp.sqrt((rmsDetNoise_f**2).subs(ini.frequency, f_max) - (rmsDetNoise_f**2).subs(ini.frequency, f_min)))

print(rmsDetNoise_fMin_fmax)

# However, during amplifier design the frequency range of interest is seldom ##
# a design parameter and numeric integration limits can be used.             ##

head2html('Detector-referred RMS noise')
text2html('The RMS noise $V_{n_{out}}$ at the detector over a frequency range from $f_{\\min}$ to $f_{\\max}$ is obtained as:')
eqn2html('V_n_out', rmsDetNoise_fMin_fmax)

# Find a symbolic expression for the source-referred RMS noise over a #########
# frequency range from f_min to f_max                                 #########
try:
    rmsSrcNoise_fMin_fmax = rmsNoise(result, 'inoise', f_min, f_max)
except:
    print('Trying the work around:\n')
    # With the work around (for this situation) ###############################
    rmsSrcNoise_f = rmsNoise(result, 'inoise', 0, ini.frequency)
    rmsSrcNoise_fMin_fmax = sp.simplify(sp.sqrt((rmsSrcNoise_f**2).subs(ini.frequency, f_max) - (rmsSrcNoise_f**2).subs(ini.frequency, f_min)))
print(rmsSrcNoise_fMin_fmax)

# Put the results on the HTML page ############################################
head2html('Source-referred RMS noise')
text2html('The source-referred RMS noise $V_{n_{src}}$ over a frequency range from $f_{\\min}$ to $f_{\\max}$ is obtained as:')
eqn2html('V_n_src', rmsSrcNoise_fMin_fmax)

# Evalueate the noise figure in dB and put it on the HTML page ################
head2html('Noise figure')
# Note: the logarithmic function in sympy has the natural base.
F = 10*sp.log(sp.simplify(rmsSrcNoise_fMin_fmax**2/rmsNoise(result, 'inoise', f_min, f_max, source='V1')**2))/sp.log(10)
text2html('The noise figure in dB is:')
eqn2html('NF', F)

# We will plot the spectrum versus frequency for R_s = 1k, R_p = 10k and C_p = 50n
# The frequency range of interest is 100Hz to 100kHz

i1.defPar('C_p', '50n')
i1.defPar('R_s', '1k')
i1.defPar('R_p', '10k')

# You can substitute these values in the sumbolically calculated 'results',
# or set the simulation type to 'numeric' and execute the instruction again.

i1.setSimType('numeric')
result = i1.execute()

# Create a figure: default function for plotting results from dataType 'noise' is 'onoise'

figOnoise = plotSweep('Onoise', 'Detector-referred noise', result, 0.1, 100, 200, sweepScale='k', show=True)

# You can also plot the contributions of 'all' or specific sources to the 
# source-referred, or detector-referred noise:

# Here we plot the contribution of all noise sources to to source rederref noise

figInoiseContribsAll = plotSweep('InoiseContribsAll', 'Contributions to the source-referred noise',      result, 0.1, 100, 200, sweepScale='k', show=True, funcType='inoise', noiseSources = 'all', yLim = [1e-18, 1e-16])
# For the next instruction you need to update SLiCAP (minor bug fix!)
figInoiseContribsV1  = plotSweep('InoiseContribsV1',  'Contribution of V1 to the source-referred noise', result, 0.1, 100, 200, sweepScale='k', show=True, funcType='inoise', noiseSources = 'V1',  yLim = [1e-18, 1e-16])

# We will plot the total RMS output noise over the frequency of interest  
# versus 'C_p'
# To this end, we create a parameter with this result.
# We can use the symbolic expression for the RMS detector noise calculated 
# above, and substitute the numeric values

R_s, R_p, k, T = sp.symbols('R_s, R_p, k, T')
i1.defPar('V_onoise', rmsDetNoise_fMin_fmax.subs([(f_min,100), (f_max, 1e5), (R_s, 1e3), (R_p, 1e4), (k, i1.getParValue('k')), (T, i1.getParValue('T'))]))
# Execute the instruction with dataType='params'
i1.setDataType('params')
result= i1.execute()
figRMSnoiseCpO1 = plotSweep('InoiseRMS_Cp', 'Detector-referred RMS noise 100Hz-100kHz',result, 1 , 500, 500, yVar='V_onoise', yUnits='V', yScale='u', sweepVar='C_p', sweepScale='n', xUnits='F', funcType='param', show=True)

# Another approach is to run the numeric simulation again after deleting the
# definition of the parameter 'C_p'

i1.delPar('C_p')
i1.setDataType('noise')
result = i1.execute()
# Calculate the RMS value of the detector referred noise over the frequency
# range of interest (just as above) and define a parameter for this result
i1.defPar('V_onoise', rmsNoise(result, 'onoise', 100 , 1e5))
# Execute the instruction with dataType='params'; redefine the sweep parameter
i1.defPar('C_p', 0) 
result= i1.execute()
figRMSnoiseCpO2 = plotSweep('InoiseRMS_Cp', 'Detector-referred RMS noise 100Hz-100kHz versus $C_p$',result, 1 , 500, 500, yVar='V_onoise', yUnits='V', yScale='u', sweepVar='C_p', sweepScale='n', xUnits='F', funcType='param', show=True)

# Finally we will put some plots on an HTML page
htmlPage('Plots')
head2html('Detector referred noise')
fig2html(figOnoise, 600, label='figOnoise', caption='Detector referred noise spectrum')
head2html('Contributions to the source-referred noise')
fig2html(figInoiseContribsAll, 600, label='figInoiseAll', caption='Contributions to the source-referred noise spectrum')
fig2html(figInoiseContribsV1,  600, label='figInoiseV1',  caption='Contribution of V1 to the source-referred noise spectrum')
head2html('Detector-referred RMS noise versus C_p')
fig2html(figRMSnoiseCpO2,  600, label='figOnoiseRMS_Cp',  caption='Detector-referred RMS noise 100Hz-100kHz versus $C_p$')